@@ -1,15 +1,19 @@
 # -*- test-case-name: twisted.mail.test.test_smtp -*-
-#
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
-"""Simple Mail Transfer Protocol implementation.
+"""
+Simple Mail Transfer Protocol implementation.
 """
 
-from __future__ import generators
+import time, re, base64, types, socket, os, random, hmac
+import MimeWriter, tempfile, rfc822
+import warnings
+import binascii
+from email.base64MIME import encode as encode_base64
+
+from zope.interface import implements, Interface
 
-# Twisted imports
 from twisted.copyright import longversion
 from twisted.protocols import basic
 from twisted.protocols import policies
@@ -19,9 +23,7 @@ from twisted.internet import error
 from twisted.internet import reactor
 from twisted.internet.interfaces import ITLSTransport
 from twisted.python import log
-from twisted.python import components
 from twisted.python import util
-from twisted.python import reflect
 from twisted.python import failure
 
 from twisted import cred
@@ -29,20 +31,6 @@ import twisted.cred.checkers
 import twisted.cred.credentials
 from twisted.python.runtime import platform
 
-# System imports
-import time, string, re, base64, types, socket, os, random, hmac
-import MimeWriter, tempfile, rfc822
-import warnings
-import binascii
-import sys
-from zope.interface import implements
-
-try:
-    from email.base64MIME import encode as encode_base64
-except ImportError:
-    def encode_base64(s, eol='\n'):
-        return s.encode('base64').rstrip() + eol
-
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -59,8 +47,8 @@ else:
 # Used for fast success code lookup
 SUCCESS = dict(map(None, range(200, 300), []))
 
-class IMessageDelivery(components.Interface):
-    def receivedHeader(self, helo, origin, recipients):
+class IMessageDelivery(Interface):
+    def receivedHeader(helo, origin, recipients):
         """
         Generate the Received header for a message
 
@@ -79,7 +67,7 @@ class IMessageDelivery(components.Interface):
         @return: The full \"Received\" header string.
         """
 
-    def validateTo(self, user):
+    def validateTo(user):
         """
         Validate the address for which the message is destined.
 
@@ -96,7 +84,7 @@ class IMessageDelivery(components.Interface):
         not to be accepted.
         """
 
-    def validateFrom(self, helo, origin):
+    def validateFrom(helo, origin):
         """
         Validate the address from which the message originates.
 
@@ -115,7 +103,7 @@ class IMessageDelivery(components.Interface):
         not to be accepted.
         """
 
-class IMessageDeliveryFactory(components.Interface):
+class IMessageDeliveryFactory(Interface):
     """An alternate interface to implement for handling message delivery.
 
     It is useful to implement this interface instead of L{IMessageDelivery}
@@ -125,7 +113,7 @@ class IMessageDeliveryFactory(components.Interface):
     something which cannot be done by L{IMessageDelivery} implementors
     due to their lack of information.
     """
-    def getMessageDelivery(self):
+    def getMessageDelivery():
         """Return an L{IMessageDelivery} object.
 
         This will be called once per message.
@@ -487,20 +475,20 @@ class User:
     def __str__(self):
         return str(self.dest)
 
-class IMessage(components.Interface):
+class IMessage(Interface):
     """Interface definition for messages that can be sent via SMTP."""
 
-    def lineReceived(self, line):
+    def lineReceived(line):
         """handle another line"""
 
-    def eomReceived(self):
+    def eomReceived():
         """handle end of message
 
         return a deferred. The deferred should be called with either:
         callback(string) or errback(error)
         """
 
-    def connectionLost(self):
+    def connectionLost():
         """handle message truncated
 
         semantics should be to discard the message
@@ -547,7 +535,7 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         self.transport.loseConnection()
 
     def greeting(self):
-        return '%s NO UCE NO UBE NO RELAY PROBES ESMTP' % (self.host,)
+        return '%s NO UCE NO UBE NO RELAY PROBES' % (self.host,)
 
     def connectionMade(self):
         # Ensure user-code always gets something sane for _helo
@@ -652,27 +640,27 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
             self.sendCode(553, str(e))
             return
 
-        defer.maybeDeferred(self.validateFrom, self._helo, addr
-            ).addCallbacks(self._cbFromValidate, self._ebFromValidate
-            )
+        validated = defer.maybeDeferred(self.validateFrom, self._helo, addr)
+        validated.addCallbacks(self._cbFromValidate, self._ebFromValidate)
+
 
     def _cbFromValidate(self, from_, code=250, msg='Sender address accepted'):
         self._from = from_
         self.sendCode(code, msg)
 
+
     def _ebFromValidate(self, failure):
         if failure.check(SMTPBadSender):
             self.sendCode(failure.value.code,
-                          'Cannot receive for specified address %s: %s'
+                          'Cannot receive from specified address %s: %s'
                           % (quoteaddr(failure.value.addr), failure.value.resp))
         elif failure.check(SMTPServerError):
             self.sendCode(failure.value.code, failure.value.resp)
         else:
-            log.err(failure)
+            log.err(failure, "SMTP sender validation failure")
             self.sendCode(
                 451,
-                'Requested action aborted: local error in processing'
-            )
+                'Requested action aborted: local error in processing')
 
 
     def do_RCPT(self, rest):
@@ -846,7 +834,11 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         else:
             self.sendCode(250, 'Delivery in progress')
 
-    def _cbAuthenticated(self, (iface, avatar, logout)):
+
+    def _cbAnonymousAuthentication(self, (iface, avatar, logout)):
+        """
+        Save the state resulting from a successful anonymous cred login.
+        """
         if issubclass(iface, IMessageDeliveryFactory):
             self.deliveryFactory = avatar
             self.delivery = None
@@ -856,18 +848,8 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         else:
             raise RuntimeError("%s is not a supported interface" % (iface.__name__,))
         self._onLogout = logout
-        self.authenticated = 1
         self.challenger = None
 
-    def _ebAuthenticated(self, reason):
-        self.challenge = None
-        if reason.check(cred.error.UnauthorizedLogin):
-            self.sendCode(535, 'Authentication failed')
-        elif reason.check(SMTPAddressError):
-            self.sendCode(reason.value.code, reason.value.resp)
-        else:
-            self.sendCode(451, 'Requested action aborted: local error in processing')
-            log.err(reason)
 
     # overridable methods:
     def validateFrom(self, helo, origin):
@@ -899,14 +881,42 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
         # provided: try to perform an anonymous login and then invoke this
         # method again.
         if self.portal:
-            return self.portal.login(cred.credentials.Anonymous(), None,
-                    IMessageDeliveryFactory, IMessageDelivery
-                ).addCallback(self._cbAuthenticated
-                ).addCallback(lambda _: self.validateFrom(helo, origin)
-                ).addErrback(self._ebAuthenticated
-                )
+
+            result = self.portal.login(
+                cred.credentials.Anonymous(),
+                None,
+                IMessageDeliveryFactory, IMessageDelivery)
+
+            def ebAuthentication(err):
+                """
+                Translate cred exceptions into SMTP exceptions so that the
+                protocol code which invokes C{validateFrom} can properly report
+                the failure.
+                """
+                if err.check(cred.error.UnauthorizedLogin):
+                    exc = SMTPBadSender(origin)
+                elif err.check(cred.error.UnhandledCredentials):
+                    exc = SMTPBadSender(
+                        origin, resp="Unauthenticated senders not allowed")
+                else:
+                    return err
+                return defer.fail(exc)
+
+            result.addCallbacks(
+                self._cbAnonymousAuthentication, ebAuthentication)
+
+            def continueValidation(ignored):
+                """
+                Re-attempt from address validation.
+                """
+                return self.validateFrom(helo, origin)
+
+            result.addCallback(continueValidation)
+            return result
+
         raise SMTPBadSender(origin)
 
+
     def validateTo(self, user):
         """
         Validate the address for which the message is destined.
@@ -936,7 +946,9 @@ class SMTP(basic.LineOnlyReceiver, policies.TimeoutMixin):
             heloStr = " helo=%s" % (helo[0],)
         domain = self.transport.getHost().host
         from_ = "from %s ([%s]%s)" % (helo[0], helo[1], heloStr)
-        by = "by %s with ESMTP (%s)" % (domain, longversion)
+        by = "by %s with %s (%s)" % (domain,
+                                     self.__class__.__name__,
+                                     longversion)
         for_ = "for %s; %s" % (' '.join(map(str, recipients)),
                                rfc822date())
         return "Received: %s\n\t%s\n\t%s" % (from_, by, for_)
@@ -1210,9 +1222,11 @@ class ESMTPClient(SMTPClient):
         self.context = contextFactory
         self.tlsMode = False
 
+
     def esmtpEHLORequired(self, code=-1, resp=None):
         self.sendError(EHLORequiredError(502, "Server does not support ESMTP Authentication", self.log.str()))
 
+
     def esmtpAUTHRequired(self, code=-1, resp=None):
         tmp = []
 
@@ -1224,6 +1238,7 @@ class ESMTPClient(SMTPClient):
         self.sendError(AUTHRequiredError(502, "Server does not support Client Authentication schemes %s" % auth,
                                          self.log.str()))
 
+
     def esmtpTLSRequired(self, code=-1, resp=None):
         self.sendError(TLSRequiredError(502, "Server does not support secure communication via TLS / SSL",
                                         self.log.str()))
@@ -1399,6 +1414,11 @@ class ESMTP(SMTP):
         self.canStartTLS = ITLSTransport.providedBy(self.transport)
         self.canStartTLS = self.canStartTLS and (self.ctx is not None)
 
+
+    def greeting(self):
+        return SMTP.greeting(self) + ' ESMTP'
+
+
     def extensions(self):
         ext = {'AUTH': self.challengers.keys()}
         if self.canStartTLS and not self.startedTLS:
@@ -1454,42 +1474,95 @@ class ESMTP(SMTP):
         if not chal:
             self.sendCode(504, 'Unrecognized authentication type')
             return
-        self.authenticate(chal)
 
-    def authenticate(self, challenger):
-        if self.portal:
-            challenge = challenger.getChallenge()
-            coded = base64.encodestring(challenge)[:-1]
-            self.sendCode(334, coded)
-            self.mode = AUTH
-            self.challenger = challenger
+        self.mode = AUTH
+        self.challenger = chal
+
+        if len(parts) > 1:
+            chal.getChallenge() # Discard it, apparently the client does not
+                                # care about it.
+            rest = parts[1]
         else:
+            rest = None
+        self.state_AUTH(rest)
+
+
+    def _cbAuthenticated(self, loginInfo):
+        """
+        Save the state resulting from a successful cred login and mark this
+        connection as authenticated.
+        """
+        result = SMTP._cbAnonymousAuthentication(self, loginInfo)
+        self.authenticated = True
+        return result
+
+
+    def _ebAuthenticated(self, reason):
+        """
+        Handle cred login errors by translating them to the SMTP authenticate
+        failed.  Translate all other errors into a generic SMTP error code and
+        log the failure for inspection.  Stop all errors from propagating.
+        """
+        self.challenge = None
+        if reason.check(cred.error.UnauthorizedLogin):
+            self.sendCode(535, 'Authentication failed')
+        else:
+            log.err(failure, "SMTP authentication failure")
+            self.sendCode(
+                451,
+                'Requested action aborted: local error in processing')
+
+
+    def state_AUTH(self, response):
+        """
+        Handle one step of challenge/response authentication.
+
+        @param response: The text of a response. If None, this
+        function has been called as a result of an AUTH command with
+        no initial response. A response of '*' aborts authentication,
+        as per RFC 2554.
+        """
+        if self.portal is None:
             self.sendCode(454, 'Temporary authentication failure')
+            self.mode = COMMAND
+            return
 
-    def state_AUTH(self, rest):
-        self.mode = COMMAND
+        if response is None:
+            challenge = self.challenger.getChallenge()
+            encoded = challenge.encode('base64')
+            self.sendCode(334, encoded)
+            return
 
-        if rest == '*':
+        if response == '*':
             self.sendCode(501, 'Authentication aborted')
-            self.challenger.abort()
             self.challenger = None
+            self.mode = COMMAND
             return
 
         try:
-            uncoded = base64.decodestring(rest)
-        except binascii.Error, e:
-            self._ebAuthenticated(failure.Failure(e))
-        else:
-            self.challenger.setResponse(uncoded)
-            if self.challenger.moreChallenges():
-                self.authenticate(self.challenger)
-            else:
-                self.portal.login(self.challenger, None,
-                        IMessageDeliveryFactory, IMessageDelivery
-                    ).addCallback(self._cbAuthenticated
-                    ).addCallback(lambda _: self.sendCode(235, 'Authentication successful.')
-                    ).addErrback(self._ebAuthenticated
-                    )
+            uncoded = response.decode('base64')
+        except binascii.Error:
+            self.sendCode(501, 'Syntax error in parameters or arguments')
+            self.challenger = None
+            self.mode = COMMAND
+            return
+
+        self.challenger.setResponse(uncoded)
+        if self.challenger.moreChallenges():
+            challenge = self.challenger.getChallenge()
+            coded = challenge.encode('base64')[:-1]
+            self.sendCode(334, coded)
+            return
+
+        self.mode = COMMAND
+        result = self.portal.login(
+            self.challenger, None,
+            IMessageDeliveryFactory, IMessageDelivery)
+        result.addCallback(self._cbAuthenticated)
+        result.addCallback(lambda ign: self.sendCode(235, 'Authentication successful.'))
+        result.addErrback(self._ebAuthenticated)
+
+
 
 class SenderMixin:
     """Utility class for sending emails easily.
@@ -1619,14 +1692,14 @@ class SMTPSenderFactory(protocol.ClientFactory):
         return p
 
 
-class IClientAuthentication(components.Interface):
-    def getName(self):
+class IClientAuthentication(Interface):
+    def getName():
         """Return an identifier associated with this authentication scheme.
 
         @rtype: C{str}
         """
 
-    def challengeResponse(self, secret, challenge):
+    def challengeResponse(secret, challenge):
         """Generate a challenge response string"""
 
 
@@ -1888,35 +1961,44 @@ def sendEmail(smtphost, fromEmail, toEmail, content, headers = None, attachments
 ## Yerg.  Codecs!
 ##
 import codecs
-def xtext_encode(s):
+def xtext_encode(s, errors=None):
     r = []
     for ch in s:
         o = ord(ch)
         if ch == '+' or ch == '=' or o < 33 or o > 126:
             r.append('+%02X' % o)
         else:
-            r.append(ch)
+            r.append(chr(o))
+    return (''.join(r), len(s))
+
+
+def _slowXTextDecode(s, errors=None):
+    """
+    Decode the xtext-encoded string C{s}.
+    """
+    r = []
+    i = 0
+    while i < len(s):
+        if s[i] == '+':
+            try:
+                r.append(chr(int(s[i + 1:i + 3], 16)))
+            except ValueError:
+                r.append(s[i:i + 3])
+            i += 3
+        else:
+            r.append(s[i])
+            i += 1
     return (''.join(r), len(s))
 
 try:
     from twisted.protocols._c_urlarg import unquote as _helper_unquote
 except ImportError:
-    def xtext_decode(s):
-        r = []
-        i = 0
-        while i < len(s):
-            if s[i] == '+':
-                try:
-                    r.append(chr(int(s[i + 1:i + 3], 16)))
-                except ValueError:
-                    r.append(s[i:i + 3])
-                i += 3
-            else:
-                r.append(s[i])
-                i += 1
-        return (''.join(r), len(s))
+    xtext_decode = _slowXTextDecode
 else:
-    def xtext_decode(s):
+    def xtext_decode(s, errors=None):
+        """
+        Decode the xtext-encoded string C{s} using a fast extension function.
+        """
         return (_helper_unquote(s, '+'), len(s))
 
 class xtextStreamReader(codecs.StreamReader):
